import numpy as np
import pandas as pd

from src.util.exceptions import ImplicitMetricValueError, InvalidMetricValue


def resolve_metric_list_parameter(metric):
    """
    Resolves the metric list parameter to calculate a measure

    This functions converts the metric parameter to a pandas Series if it is a list (CalculateMeasures endpoint)
    otherwise it just returns the metric - already a pandas Series (Analysis endpoint).
    """
    return (
        pd.Series(metric, dtype=np.float64)
        if isinstance(metric, (list, np.ndarray))
        else metric
    )


def get_non_complex_files_density(
    data: dict,
):
    """
    Calculates non-complex files density (em1).

    This function calculates non-complex files density measure (em1)
    used to assess the changeability quality sub characteristic.
    """

    files_complexity = resolve_metric_list_parameter(data.get("complexity"))
    files_functions = resolve_metric_list_parameter(data.get("functions"))

    if len(files_complexity) != len(files_functions):
        raise ImplicitMetricValueError(
            (
                "Unable to get the implicit value of metric `number_of_file` "
                "because the size of the lists of values of metrics `complexity` "
                "and `functions` are not equal."
            )
        )
    else:
        number_of_files = data.get("number_of_files", len(files_complexity))

    has_none = files_complexity is None or files_functions is None
    has_zero = len(files_complexity) == 0 or len(files_functions) == 0

    if has_none or has_zero:
        return 0

    if files_complexity.sum() <= 0:
        raise InvalidMetricValue(
            "The cyclomatic complexity of all files is lesser or equal than 0"
        )

    if files_functions.sum() <= 0:
        raise InvalidMetricValue(
            "The number of functions of all files is lesser or equal than 0"
        )

    complex_files_density = np.array(files_complexity / files_functions)

    return complex_files_density, number_of_files


def get_commented_files_density(data: dict):
    """
    Calculates commented files density (em2).

    This function calculates commented files density measure (em2)
    used to assess the changeability quality sub characteristic.
    """

    files_comment_lines_density = resolve_metric_list_parameter(
        data["comment_lines_density"]
    )

    number_of_files = data.get("number_of_files", len(files_comment_lines_density))

    has_none = files_comment_lines_density is None
    has_zero = len(files_comment_lines_density) == 0

    if has_none or has_zero:
        return 0, 0

    if files_comment_lines_density.sum() < 0:
        raise InvalidMetricValue(
            "The number of files comment lines density is lesser than 0"
        )

    return files_comment_lines_density, number_of_files


def get_absence_of_duplications(
    data: dict,
):
    """
    Calculates duplicated files absence (em3).

    This function calculates the duplicated files absence measure (em3)
    used to assess the changeability quality sub characteristic.
    """

    files_duplicated_lines_density = resolve_metric_list_parameter(
        data["duplicated_lines_density"]
    )

    number_of_files = data.get("number_of_files", len(files_duplicated_lines_density))

    has_none = files_duplicated_lines_density is None
    has_zero = len(files_duplicated_lines_density) == 0

    if has_none or has_zero:
        return 0

    if files_duplicated_lines_density.sum() < 0:
        raise InvalidMetricValue(
            "The number of files duplicated lines density is lesser than 0"
        )

    return files_duplicated_lines_density, number_of_files


def get_passed_tests(data: dict[str, float]):
    """
    Calculates passed tests (em4)

    This function calculates the passed tests measure (em4)
    used to assess the testing status sub characteristic.
    """

    # number_of_tests está retornando valores incorretos
    number_of_tests = resolve_metric_list_parameter(data["tests"]).sum()
    number_of_test_errors = data["test_errors"]
    number_of_test_failures = data["test_failures"]

    number_of_fail_tests = number_of_test_errors + number_of_test_failures
    return number_of_tests, number_of_fail_tests


def get_fast_test_builds(data: dict[str, list]):
    """
    Calculates fast test builds (em5)

    This function calculates the fast test builds measure (em5)
    used to assess the testing status sub characteristic.
    """

    execution_time = resolve_metric_list_parameter(data["test_execution_time"])
    number_of_tests = resolve_metric_list_parameter(data["tests"])
    number_of_files = len(execution_time)

    has_none = execution_time is None or number_of_tests is None
    has_zero = len(execution_time) == 0 or len(number_of_tests) == 0

    if has_none or has_zero:
        return 0, 0, 0

    return execution_time, number_of_files, number_of_tests


def get_test_coverage(data: dict):
    """
    Calculates test coverage (em6).

    This function calculates the test coverage measure (em6)
    used to assess the testing status sub characteristic.
    """

    coverage = resolve_metric_list_parameter(data["coverage"])

    if "number_of_files" in data:
        number_of_files = data["number_of_files"]
    else:
        number_of_files = len(coverage)

    has_none = coverage is None
    has_zero = len(coverage) == 0

    if has_none or has_zero:
        return 0

    return coverage, number_of_files


def get_team_throughput(data: dict[str, int]):
    """
    Calculates team throughput (em7)

    This function calculates the team throughput measure (em7)
    used to assess the testing status sub characteristic.
    """

    total_issues = data["total_issues"]
    resolved_issues = data["resolved_issues"]

    return 100 * (resolved_issues / total_issues)



from marshmallow import Schema, fields, validate
from marshmallow.exceptions import ValidationError


class MetricSchema(Schema):
    """
    {
        "key": "tests"
        "value": [10.0]
    }
    """

    key = fields.Str(required=True)
    value = fields.List(fields.Float(required=True))


class MeasureSchema(Schema):
    """
    {
        "key": "passed_tests",
        "metrics": [
            {
                "key": "tests",
                "value": [10.0]
            },
            {
                "key": "test_errors",
                "value": [3.0]
            },
            {
                "key": "test_failures",
                "value": [1.0]
            }
        ]
    }
    """

    key = fields.Str(required=True)
    metrics = fields.List(fields.Nested(MetricSchema), required=True)


class CalculateMeasureSchema(Schema):
    """
    {

    "measures": [
            {
                "key": "passed_tests",
                "metrics": [
                    {
                        "key": "tests",
                        "value": [10.0]
                    },
                    {
                        "key": "test_errors",
                        "value": [3.0]
                    },
                    {
                        "key": "test_failures",
                        "value": [1.0]
                    }
                ]
            },
            {
                "key": "test_builds",
                "metrics": [
                    {
                        "key": "test_execution_time",
                        "value": [8.0]
                    },
                    {
                        "key": "tests",
                        "value": [10.0]
                    }
                ]
            },
    }
    """

    measures = fields.List(fields.Nested(MeasureSchema), required=True)


class CalculatedSubEntitySchema(Schema):
    key = fields.Str(required=True)
    value = fields.Float(validate=validate.Range(min=0, max=1), required=True)
    weight = fields.Float(validate=validate.Range(min=0, max=100), required=True)


class SubCharacteristicSchema(Schema):
    key = fields.Str(required=True)
    measures = fields.List(fields.Nested(CalculatedSubEntitySchema), required=True)


class CalculateSubCharacteristicSchema(Schema):
    """
    {
        "subcharacteristics": [
            {
                "key": "testing_status",
                "measures": [
                    {
                        "key": "passed_tests",
                        "value": 1.0,
                        "weight": 33,
                    },
                    ...
                ]
            },
            ...
        ]
    }
    """

    subcharacteristics = fields.List(
        fields.Nested(SubCharacteristicSchema), required=True
    )


class CharacteristicSchema(Schema):
    key = fields.Str(required=True)
    subcharacteristics = fields.List(
        fields.Nested(CalculatedSubEntitySchema), required=True
    )


class CalculateCharacteristicSchema(Schema):
    """
    {
        "characteristics": [
            {
                "key": "reliability",
                "subcharacteristics": [
                    {
                        "key": "testing_status",
                        "value": 1.0,
                        "weight": 50,
                    },
                    ...
                ]
            },
            ...
        ]
    }
    """

    characteristics = fields.List(fields.Nested(CharacteristicSchema), required=True)


class TSQMISchema(Schema):
    key = fields.Str(required=True)
    characteristics = fields.List(
        fields.Nested(CalculatedSubEntitySchema), required=True
    )


class CalculateTSQMISchema(Schema):
    """
    {
        "tsqmi": {
            "key": "tsqmi",
            "characteristics": [
                {
                    "key": "reliability",
                    "value": 1.0,
                    "weight": 50,
                },
                ...
            ]
        }
    }
    """

    tsqmi = fields.Nested(TSQMISchema, required=True)


class NonComplexFileDensitySchema(Schema):
    # 1 Validação : Se contém uma lista de métricas
    metrics = fields.List(fields.Nested(MetricSchema), required=True)

    @staticmethod
    def validate_metrics(metrics):
        for metric in metrics:
            # 2 Validação : Se foi passada alguma métrica não pertencente a medida
            if metric["key"] not in ["complexity", "functions"]:
                raise ValidationError(
                    f"'{metric['key']}': Métrica não presente na medida"
                )


class CommentedFileDensitySchema(Schema):
    # 1 Validação : Se contém uma lista de métricas
    metrics = fields.List(fields.Nested(MetricSchema), required=True)

    @staticmethod
    def validate_metrics(metrics):
        for metric in metrics:
            # 2 Validação : Se foi passada alguma métrica não pertencente a medida
            if metric["key"] not in ["comment_lines_density"]:
                raise ValidationError(
                    f"'{metric['key']}': Métrica não presente na medida"
                )


class DuplicationAbsenceSchema(Schema):
    # 1 Validação : Se contém uma lista de métricas
    metrics = fields.List(fields.Nested(MetricSchema), required=True)

    @staticmethod
    def validate_metrics(metrics):
        for metric in metrics:
            # 2 Validação : Se foi passada alguma métrica não pertencente a medida
            if metric["key"] not in ["duplicated_lines_density"]:
                raise ValidationError(
                    f"'{metric['key']}': Métrica não presente na medida"
                )


class PassedTestsSchema(Schema):
    # 1 Validação : Se contém uma lista de métricas
    metrics = fields.List(fields.Nested(MetricSchema), required=True)

    @staticmethod
    def validate_metrics(metrics):
        for metric in metrics:
            # 2 Validação : Se foi passada alguma métrica não pertencente a medida
            if metric["key"] not in ["tests", "test_failures", "test_errors"]:
                raise ValidationError(
                    f"'{metric['key']}': Métrica não presente na medida"
                )

            # 3 Validação: As métricas test_failures e test_errors só podem ser
            #             representadas por um array de um único elemento flutuante
            if metric["key"] in ["test_failures", "test_errors"]:
                if len(metric["value"]) != 1:
                    raise ValidationError(
                        f"'{metric['key']}': Deveria ser um array de um único valor"
                    )
                if not isinstance(metric["value"][0], float):
                    raise ValidationError(
                        f"'{metric['key']}': Deveria ser um valor flutuante"
                    )


class TestBuildsSchema(Schema):
    # 1 Validação : Se contém uma lista de métricas
    metrics = fields.List(fields.Nested(MetricSchema), required=True)

    @staticmethod
    def validate_metrics(metrics):
        for metric in metrics:
            # 2 Validação : Se foi passada alguma métrica não pertencente a medida
            if metric["key"] not in ["test_execution_time", "tests"]:
                raise ValidationError(
                    f"'{metric['key']}': Métrica não presente na medida"
                )


class TestCoverageSchema(Schema):
    # 1 Validação : Se contém uma lista de métricas
    metrics = fields.List(fields.Nested(MetricSchema), required=True)

    @staticmethod
    def validate_metrics(metrics):
        for metric in metrics:
            # 2 Validação : Se foi passada alguma métrica não pertencente a medida
            if metric["key"] not in ["coverage"]:
                raise ValidationError(
                    f"'{metric['key']}': Métrica não presente na medida"
                )


class TeamThroughputSchema(Schema):
    # 1 Validação : Se contém uma lista de métricas
    metrics = fields.List(fields.Nested(MetricSchema), required=True)

    @staticmethod
    def validate_metrics(metrics):
        for metric in metrics:
            # 2 Validação : Se foi passada alguma métrica não pertencente a medida
            if metric["key"] not in ["resolved_issues", "total_issues"]:
                raise ValidationError(
                    f"'{metric['key']}': Métrica não presente na medida"
                )

            # 3 Validação: As métricas só podem ser representadas por um array de
            #             um único elemento inteiro
            if len(metric["value"]) != 1:
                raise ValidationError(
                    f"'{metric['key']}': Deveria ser um array de um único valor"
                )
            if not isinstance(metric["value"][0], int):
                raise ValidationError(
                    f"'{metric['key']}': Deveria ser um valor inteiro"
                )



import math

import numpy as np

from src.util.exceptions import (
    InvalidCheckThreshold,
    InvalidMetricValue,
    InvalidThresholdValue,
    ValuesAndWeightsOfDifferentSizes,
)


class Checker:
    @staticmethod
    def check_non_complex_files_density_threshold(
        min_threshold: float, max_threshold: float
    ):
        if min_threshold != 0:
            raise InvalidThresholdValue("min_threshold is not equal to 0")

        if min_threshold >= max_threshold:
            raise InvalidThresholdValue(
                "min_threshold is greater or equal to max_threshold"
            )

    @staticmethod
    def check_comment_files_density_threshold(
        min_threshold: float, max_threshold: float
    ):
        if min_threshold < 0:
            raise InvalidThresholdValue("min_threshold is lesser than 0")
        if min_threshold >= max_threshold:
            raise InvalidThresholdValue(
                "min_threshold is greater or equal to max_threshold"
            )
        if max_threshold > 100:
            raise InvalidThresholdValue("max_threshold is greater than 100")

    @staticmethod
    def check_absence_of_duplications_threshold(
        min_threshold: float, max_threshold: float
    ):
        if min_threshold != 0:
            raise InvalidThresholdValue("min_threshold is not equal to 0")

        if min_threshold >= max_threshold:
            raise InvalidThresholdValue(
                "min_threshold is greater or equal to max_threshold"
            )

        if max_threshold > 100:
            raise InvalidThresholdValue("max_threshold is greater than 100")

    @staticmethod
    def check_test_coverage_threshold(min_threshold, max_threshold):
        if min_threshold < 0:
            raise InvalidThresholdValue("min_threshold is lesser than 0")

        if min_threshold >= max_threshold:
            raise InvalidThresholdValue(
                "min_threshold is greater or equal to max_threshold"
            )

        if max_threshold != 100:
            raise InvalidThresholdValue("max_threshold is not equal to 100")

    @staticmethod
    def check_fast_test_builds_threshold(min_threshold: float, max_threshold: float):
        if min_threshold != 0:
            raise InvalidThresholdValue(("min_threshold is not equal to 0"))

        if min_threshold >= max_threshold:
            raise InvalidThresholdValue(
                "min_threshold is greater or equal to max_threshold"
            )

    @staticmethod
    def check_passed_tests_threshold(min_threshold: float, max_threshold: float):
        if min_threshold != 0:
            raise InvalidThresholdValue("min_threshold is not equal to 0")

        if max_threshold != 1:
            raise InvalidThresholdValue("max_threshold is not equal to 1")

    @staticmethod
    def check_threshold(min: float, max: float, measure: str):
        check_function = getattr(Checker, f"check_{measure}_threshold", None)
        if check_function is None:
            raise InvalidCheckThreshold("Check function not implemented")
        check_function(min, max)

    @staticmethod
    def check_metric_value(metric_value, metric):
        try:
            if metric_value is None or math.isnan(float(metric_value)):
                raise InvalidMetricValue(f'"{metric}" has an invalid metric value')
        except (ValueError, TypeError):
            raise InvalidMetricValue(f'"{metric}" has an invalid metric value')

    @staticmethod
    def check_metric_values(metric_values, metric):
        if not isinstance(metric_values, (list, np.ndarray)):
            raise InvalidMetricValue(f'"{metric}" is not a list or a np.ndarray')

        # if not len(metric_values):
        #     raise InvalidMetricValue(f'"{metric}" is empty')

        for value in metric_values:
            Checker.check_metric_value(value, metric)

    @staticmethod
    def check_aggregated_weighted_values(values, weights):
        if len(values) != len(weights):
            raise ValuesAndWeightsOfDifferentSizes(
                "The length of weight and values are not equal",
            )

    @staticmethod
    def check_team_throughput_threshold(min_threshold: float, max_threshold: float):
        if min_threshold != 45:
            raise InvalidThresholdValue("min_threshold is not equal to 45")

        if max_threshold != 100:
            raise InvalidThresholdValue("max_threshold is not equal to 100")



import numpy as np

import src.core.measures_functions as ems_functions
import src.core.transformations as transformations
from src.util.check import Checker


def non_complex_files_density(
    data_frame,
    min_threshold: float = 0,
    max_threshold: float = 10,
):
    """
    Calculates non-complex files density.
    This function calculates non-complex files density measure
    used to assess the changeability quality subcharacteristic.

    This function gets the dataframe metrics
    and returns the non-complex files density measure.
    """
    files_complexity = np.array(data_frame["complexity"])
    files_functions = np.array(data_frame["functions"])

    Checker.check_metric_values(files_complexity, "complexity")
    Checker.check_metric_values(files_functions, "functions")

    if len(files_complexity) == len(files_functions) == 0:
        return 0.0

    Checker.check_threshold(
        min_threshold,
        max_threshold,
        "non_complex_files_density",
    )

    (
        complex_files_density,
        number_of_files,
    ) = ems_functions.get_non_complex_files_density(
        data={
            "complexity": files_complexity,
            "functions": files_functions,
        }
    )

    files_in_thresholds_bool_index = complex_files_density <= max_threshold
    files_functions_gt_zero_bool_index = files_functions > 0
    x = complex_files_density[
        files_in_thresholds_bool_index * files_functions_gt_zero_bool_index
    ]

    interpretation_function_value = transformations.interpretation_function(
        x=x,
        min_threshold=min_threshold,
        max_threshold=max_threshold,
        gain_interpretation=-1,
    )

    aggregated_and_normalized_measure = transformations.calculate_measure(
        interpretation_function_value, number_of_files
    )
    return aggregated_and_normalized_measure


def commented_files_density(
    data_frame, min_threshold: float = 10, max_threshold: float = 30
):
    """
    Calculates commented files density.

    This function gets the dataframe metrics
    and returns the commented files density measure.
    """
    files_comment_lines_density = data_frame["comment_lines_density"]

    Checker.check_metric_values(files_comment_lines_density, "comment_lines_density")

    if len(files_comment_lines_density) == 0:
        return 0.0

    Checker.check_threshold(min_threshold, max_threshold, "comment_files_density")

    (
        files_comment_lines_density,
        number_of_files,
    ) = ems_functions.get_commented_files_density(
        data={
            "comment_lines_density": files_comment_lines_density,
        }
    )

    x = files_comment_lines_density[
        files_comment_lines_density.between(
            min_threshold,
            max_threshold,
            inclusive="both",
        )
    ]
    interpretation_function_value = transformations.interpretation_function(
        x=x,
        min_threshold=min_threshold,
        max_threshold=max_threshold,
        gain_interpretation=-1,
    )
    aggregated_and_normalized_measure = transformations.calculate_measure(
        interpretation_function_value, number_of_files
    )
    return aggregated_and_normalized_measure


def absence_of_duplications(
    data_frame, min_threshold: float = 0, max_threshold: float = 5.0
):
    """
    Calculates duplicated files absence (em3).

    This function gets the dataframe metrics
    and returns the duplicated files absence measure (em3).
    """
    files_duplicated_lines_density = data_frame["duplicated_lines_density"]  # m5 metric

    Checker.check_metric_values(
        files_duplicated_lines_density, "duplicated_lines_density"
    )

    if len(files_duplicated_lines_density) == 0:
        return 0.0

    Checker.check_threshold(min_threshold, max_threshold, "absence_of_duplications")

    (
        files_duplicated_lines_density,
        number_of_files,
    ) = ems_functions.get_absence_of_duplications(
        data={"duplicated_lines_density": files_duplicated_lines_density},
    )

    x = files_duplicated_lines_density[files_duplicated_lines_density <= max_threshold]

    interpretation_function_value = transformations.interpretation_function(
        x=x,
        min_threshold=min_threshold,
        max_threshold=max_threshold,
        gain_interpretation=-1,
    )
    aggregated_and_normalized_measure = transformations.calculate_measure(
        interpretation_function_value, number_of_files
    )
    return aggregated_and_normalized_measure


def test_coverage(
    data_frame,
    min_threshold: float = 60,
    max_threshold: float = 100,
):
    """
    Calculates test coverage (em6).

    This function gets the dataframe metrics
    and returns the test coverage measure (em6).
    """
    coverage = data_frame["coverage"]  # m6 metric

    Checker.check_metric_values(coverage, "coverage")

    if len(coverage) == 0:
        return 0.0

    Checker.check_threshold(min_threshold, max_threshold, "test_coverage")

    coverage, number_of_files = ems_functions.get_test_coverage(
        data={"coverage": coverage},
    )
    x = coverage[coverage >= min_threshold]
    interpretation_function_value = transformations.interpretation_function(
        x=x,
        min_threshold=min_threshold,
        max_threshold=max_threshold,
        gain_interpretation=1,
    )
    aggregated_and_normalized_measure = transformations.calculate_measure(
        interpretation_function_value, number_of_files
    )
    return aggregated_and_normalized_measure


def fast_test_builds(
    data_frame, min_threshold: float = 0, max_threshold: float = 300000
):
    """
    Calculates fast test builds (em5)
    This function gets the dataframe metrics
    and returns the fast test builds measure (em5).
    """
    test_execution_time = data_frame["test_execution_time"]
    tests = data_frame["tests"]

    Checker.check_metric_values(test_execution_time, "test_execution_time")
    Checker.check_metric_values(tests, "tests")

    if len(test_execution_time) == len(tests) == 0:
        return 0.0

    Checker.check_threshold(min_threshold, max_threshold, "fast_test_builds")

    (
        execution_time,
        number_of_files,
        number_of_tests,
    ) = ems_functions.get_fast_test_builds(
        data={"test_execution_time": test_execution_time, "tests": tests},
    )

    execution_between_thresholds = execution_time[execution_time <= max_threshold]
    x = np.divide(execution_between_thresholds, number_of_tests)

    interpretation_function_value = transformations.interpretation_function(
        x=x,
        min_threshold=min_threshold,
        max_threshold=max_threshold,
        gain_interpretation=-1,
    )

    aggregated_and_normalized_measure = transformations.calculate_measure(
        interpretation_function_value, number_of_files
    )

    return aggregated_and_normalized_measure


def passed_tests(data_frame, min_threshold: float = 0, max_threshold: float = 1):
    """
    Calculates passed tests (em4)

    This function gets the dataframe metrics
    and returns the passed tests measure (em4).
    """

    tests = data_frame["tests"]  # m6 metrics
    test_errors = data_frame["test_errors"]  # m7 metrics
    test_failures = data_frame["test_failures"]  # m8 metrics

    Checker.check_metric_values(tests, "tests")
    if len(tests) == 0:
        return 0.0
    Checker.check_metric_value(test_failures, "test_failures")
    Checker.check_metric_value(test_errors, "test_errors")

    Checker.check_threshold(min_threshold, max_threshold, "passed_tests")

    number_of_tests, number_of_fail_tests = ems_functions.get_passed_tests(
        data={
            "tests": tests,
            "test_errors": float(test_errors),
            "test_failures": float(test_failures),
        }
    )

    x = np.divide((number_of_tests - number_of_fail_tests), number_of_tests)

    interpretation_function_value = transformations.interpretation_function(
        x=x,
        min_threshold=min_threshold,
        max_threshold=max_threshold,
        gain_interpretation=1,
    )

    aggregated_and_normalized_measure = transformations.calculate_measure(
        interpretation_function_value
    )

    return aggregated_and_normalized_measure


def team_throughput(
    data_frame,
    min_threshold: float = 45,
    max_threshold: float = 100,
):
    """
    Calculates team throughput (em7).

    This function gets the dataframe metrics
    and returns the team throughput measure (em7).
    """
    total_issues = data_frame["total_issues"]
    resolved_issues = data_frame["resolved_issues"]

    Checker.check_metric_value(total_issues, "total_issues")
    Checker.check_metric_value(resolved_issues, "resolved_issues")

    Checker.check_threshold(min_threshold, max_threshold, "team_throughput")

    team_throughput_value = ems_functions.get_team_throughput(
        data={
            "total_issues": int(total_issues),
            "resolved_issues": int(resolved_issues),
        }
    )

    interpretation_function_value = transformations.interpretation_function(
        x=team_throughput_value,
        min_threshold=min_threshold,
        max_threshold=max_threshold,
        gain_interpretation=1,
    )

    aggregated_and_normalized_measure = transformations.calculate_measure(
        interpretation_function_value
    )

    return aggregated_and_normalized_measure



from src.core import schemas
from src.core.aggregated_normalized_measures import (
    absence_of_duplications,
    commented_files_density,
    fast_test_builds,
    non_complex_files_density,
    passed_tests,
    test_coverage,
    team_throughput,
)

AVAILABLE_PRE_CONFIGS = {
    "characteristics": {
        "reliability": {
            "name": "Reliability",
            "subcharacteristics": ["testing_status"],
        },
        "maintainability": {
            "name": "Maintainability",
            "subcharacteristics": ["modifiability"],
        },
        "functional_suitability": {
            "name": "Functional Suitability",
            "subcharacteristics": ["functional_completeness"],
        },
    },
    "subcharacteristics": {
        "testing_status": {
            "name": "Testing Status",
            "measures": [
                "passed_tests",
                "test_builds",
                "test_coverage",
            ],
            "characteristics": ["reliability"],
        },
        "modifiability": {
            "name": "Modifiability",
            "measures": [
                "non_complex_file_density",
                "commented_file_density",
                "duplication_absense",
            ],
            "characteristics": ["maintainability"],
        },
        "functional_completeness": {
            "name": "Functional Completeness",
            "measures": ["team_throughput"],
            "characteristics": ["functional_suitability"],
        },
    },
    "measures": {
        "passed_tests": {
            "name": "Passed Tests",
            "subcharacteristics": ["testing_status"],
            "characteristics": ["reliability"],
            "metrics": ["tests", "test_errors", "test_failures"],
        },
        "test_builds": {
            "name": "Test Builds",
            "subcharacteristics": ["testing_status"],
            "characteristics": ["reliability"],
            "metrics": ["tests", "test_execution_time"],
        },
        "test_coverage": {
            "name": "Test Coverage",
            "subcharacteristics": ["testing_status"],
            "characteristics": ["reliability"],
            "metrics": ["coverage"],
        },
        "non_complex_file_density": {
            "name": "Non complex file density",
            "subcharacteristics": ["modifiability"],
            "characteristics": ["maintainability"],
            "metrics": ["complexity", "functions"],
        },
        "commented_file_density": {
            "name": "Commented file density",
            "subcharacteristics": ["modifiability"],
            "characteristics": ["maintainability"],
            "metrics": ["comment_lines_density"],
        },
        "duplication_absense": {
            "name": "Duplication abscense",
            "subcharacteristics": ["modifiability"],
            "characteristics": ["maintainability"],
            "metrics": ["duplicated_lines_density"],
        },
        "team_throughput": {
            "name": "Team Throughput",
            "subcharacteristics": ["functional_completeness"],
            "characteristics": ["functional_suitability"],
            "metrics": [
                "total_issues",
                "resolved_issues",
            ],
        },
    },
}

AGGREGATED_NORMALIZED_MEASURES_MAPPING = {
    "non_complex_file_density": {
        "aggregated_normalized_measure": non_complex_files_density,
        "schema": schemas.NonComplexFileDensitySchema,
    },
    "commented_file_density": {
        "aggregated_normalized_measure": commented_files_density,
        "schema": schemas.CommentedFileDensitySchema,
    },
    "duplication_absense": {
        "aggregated_normalized_measure": absence_of_duplications,
        "schema": schemas.DuplicationAbsenceSchema,
    },
    "passed_tests": {
        "aggregated_normalized_measure": passed_tests,
        "schema": schemas.PassedTestsSchema,
    },
    "test_builds": {
        "aggregated_normalized_measure": fast_test_builds,
        "schema": schemas.TestBuildsSchema,
    },
    "test_coverage": {
        "aggregated_normalized_measure": test_coverage,
        "schema": schemas.TestCoverageSchema,
    },
    "team_throughput": {
        "aggregated_normalized_measure": team_throughput,
        "schema": schemas.TeamThroughputSchema,
    },
}



# flake8: noqa
# pylint: skip-file
DEFAULT_PRE_CONFIG = {
    "characteristics": [
        {
            "key": "reliability",
            "weight": 34,
            "subcharacteristics": [
                {
                    "key": "testing_status",
                    "weight": 100,
                    "measures": [
                        {
                            "key": "passed_tests",
                            "weight": 33,
                            "min_threshold": 0,
                            "max_threshold": 1,
                        },
                        {
                            "key": "test_builds",
                            "weight": 33,
                            "min_threshold": 0,
                            "max_threshold": 300000,
                        },
                        {
                            "key": "test_coverage",
                            "weight": 34,
                            "min_threshold": 60,
                            "max_threshold": 100,
                        },
                    ],
                }
            ],
        },
        {
            "key": "maintainability",
            "weight": 33,
            "subcharacteristics": [
                {
                    "key": "modifiability",
                    "weight": 100,
                    "measures": [
                        {
                            "key": "non_complex_file_density",
                            "weight": 33,
                            "min_threshold": 0,
                            "max_threshold": 10,
                        },
                        {
                            "key": "commented_file_density",
                            "weight": 33,
                            "min_threshold": 10,
                            "max_threshold": 30,
                        },
                        {
                            "key": "duplication_absense",
                            "weight": 34,
                            "min_threshold": 0,
                            "max_threshold": 5,
                        },
                    ],
                }
            ],
        },
        {
            "key": "functional_suitability",
            "weight": 33,
            "subcharacteristics": [
                {
                    "key": "functional_completeness",
                    "weight": 100,
                    "measures": [
                        {
                            "key": "team_throughput",
                            "weight": 100,
                            "min_threshold": 45,
                            "max_threshold": 100,
                        },
                    ],
                }
            ],
        },
    ]
}
